from ppatch.app import logger
from ppatch.commands.symbol import getsymbol_from_patch
from ppatch.config import settings
from ppatch.model import ApplyResult, Change, Hunk, Line
from ppatch.utils.common import find_list_positions


def apply_change(
    hunk_list: list[Hunk],
    target: list[Line],
    reverse: bool = False,
    flag: bool = False,
    trace: bool = False,
    flag_hunk_list: list[int] = None,
    fuzz: int = 0,
    symbols: list[str] = None,
    patch_path: str = "",
    extra_flag_hunks: list[int] = None,
) -> ApplyResult:
    """Apply a diff to a target string."""

    flag_hunk_list = [] if flag_hunk_list is None else flag_hunk_list

    if fuzz > settings.max_diff_lines or fuzz < 0:
        raise Exception(f"fuzz value should be less than {settings.max_diff_lines}")

    # TODO: 注意，修改了该函数后，需要将此处修改为对 hunk 内的 change 进行修改
    if reverse:
        if flag:
            raise Exception("flag is not supported with reverse")

        for hunk in hunk_list:
            for change in hunk.context + hunk.middle + hunk.post:
                change.old, change.new = change.new, change.old

    # 然后对每个hunk进行处理，添加偏移
    changes: list[Change] = []
    failed_hunk_list: list[Hunk] = []
    last_pos = None

    last_offset = 0
    line_count_diff = 0
    for hunk in hunk_list:

        current_hunk_fuzz = 0

        while current_hunk_fuzz <= fuzz:

            # hunk.context = hunk.context[1:]
            # hunk.post = hunk.post[: fuzz - current_hunk_fuzz]

            logger.debug(
                f"current_fuzz: {current_hunk_fuzz} len(hunk.context): {len(hunk.context)} len(hunk.post): {len(hunk.post)}"
            )

            changes_to_search = hunk.context + hunk.middle + hunk.post
            pos_list = find_list_positions(
                [line.content for line in target],
                [change.line for change in changes_to_search if change.old is not None],
            )

            if len(pos_list) != 0:
                break

            current_hunk_fuzz += 1

            if current_hunk_fuzz <= fuzz:
                hunk.context = hunk.context[1:]
                hunk.post = hunk.post[: 3 - current_hunk_fuzz]

        # 初始位置是 context 的第一个
        # 注意，前几个有可能是空
        pos_origin = None
        for change in changes_to_search:
            if change.old is not None:
                pos_origin = change.old
                break

        # TODO: 这里不太对，要想一下怎么处理，不应该是加入 failed hunk list
        # 仅在 -F 3 且只有添加行 的情况下出现（指与 GNU patch 行为不一致）
        # 也可以看一下这样的情况有多少
        if current_hunk_fuzz == fuzz and not pos_origin:
            # failed_hunk_list.append(hunk)
            # logger.debug(f"Could not determine pos_origin")
            # logger.warning(f"Apply failed with hunk {hunk.index}")
            # continue
            for change in changes_to_search:
                if change.new is not None:
                    pos_origin = change.new
                    break

            # 使用上一次偏移加上行数变化差值
            min_offset = last_offset
        else:
            if len(pos_list) == 0:
                failed_hunk_list.append(hunk)
                logger.debug(f"Could not determine proper position")
                logger.warning(f"Apply failed with hunk {hunk.index}")
                continue

            offset_list = [
                pos + 1 - pos_origin for pos in pos_list
            ]  # 确认这里是否需要 1？

            # 计算最小 offset
            min_offset = None
            for offset in offset_list:
                if min_offset is None or abs(offset) < abs(min_offset):
                    min_offset = offset

            if reverse:
                min_offset += line_count_diff
                pos_origin -= line_count_diff

        last_offset = min_offset

        # 更新行数变化差值
        hunk_add_count = sum(
            1 for c in changes_to_search if c.old is None and c.new is not None
        )
        hunk_del_count = sum(
            1 for c in changes_to_search if c.new is None and c.old is not None
        )
        line_count_diff += hunk_del_count - hunk_add_count

        logger.info(
            f"Apply hunk {hunk.index} with offset {min_offset} fuzz {current_hunk_fuzz} line_diff {line_count_diff}"
        )

        pos_new = pos_origin + min_offset - 1
        # 处理 pos_new 小于 last_pos 的情况
        logger.debug(f"pos_origin: {pos_origin}, last_pos: {last_pos}")
        if last_pos is None:
            last_pos = pos_new
        elif pos_new < last_pos:
            # 特别主要 pos_new 小于 last_pos 的情况
            logger.warning(f"Apply failed with hunk {hunk.index}")
            logger.error(f"pos: {pos_new} is greater than last_pos: {last_pos}")
            failed_hunk_list.append(hunk)
            continue
        else:
            last_pos = pos_new

        # 如果 reverse 为 True，则直接替换，不进行 flag 追踪
        if reverse:
            # 直接按照 pos 进行替换
            # 选择 offset 最小的 pos
            # pos_new = pos_origin + min_offset - 1 # 移动到上面

            old_lines = [
                change.line
                for change in hunk.context + hunk.middle + hunk.post
                if change.old is not None
            ]
            new_lines = [
                change.line
                for change in hunk.context + hunk.middle + hunk.post
                if change.new is not None
            ]

            # 检查 pos_new 位置的行是否和 old_lines 一致
            for i in range(len(old_lines)):
                if target[pos_new + i].content != old_lines[i]:
                    raise Exception(
                        f'line {pos_new + i}, "{target[pos_new + i].content}" does not match "{old_lines[i]}"'
                    )

            # 以切片的方式进行替换
            target = (
                target[:pos_new]
                + [
                    Line(index=pos_new + i, content=new_lines[i])
                    for i in range(len(new_lines))
                ]
                + target[pos_new + len(old_lines) :]
            )

        else:
            for change in hunk.middle:
                changes.append(
                    Change(
                        hunk=change.hunk,
                        old=change.old + min_offset if change.old is not None else None,
                        new=change.new + min_offset if change.new is not None else None,
                        line=change.line,
                    )
                )

    if reverse:
        return ApplyResult(
            new_line_list=target,
            conflict_hunk_num_list=[],
            failed_hunk_list=failed_hunk_list,
        )

    # 注意这里的 changes 应该使用从 hunk_list 中拼接出来的（也就是修改过行号的）
    for change in changes:
        if change.old is not None and change.line is not None:
            if change.old > len(target):
                raise Exception(
                    f'context line {change.old}, "{change.line}" does not exist in source'
                )
            if target[change.old - 1].content != change.line:
                raise Exception(
                    f'context line {change.old}, "{change.line}" does not match "{target[change.old - 1]}"'
                )

    add_count = 0
    del_count = 0

    conflict_hunk_num_list: list[int] = []
    if extra_flag_hunks is not None:  # TODO: 确认这里是否可以放在这里
        conflict_hunk_num_list += extra_flag_hunks

    for change in changes:
        # 只修改新增行和删除行（只有这些行是被修改的）
        if change.old is None and change.new is not None:
            target.insert(
                change.new - 1,
                Line(
                    index=change.new - 1,
                    content=change.line,
                    changed=True,
                    status=True,
                    flag=True if change.hunk in flag_hunk_list and flag else False,
                    hunk=change.hunk,
                ),
            )
            add_count += 1

        elif change.new is None and change.old is not None:
            index = change.old - 1 - del_count + add_count

            # 如果被修改行有标记，则标记将其删除的 hunk
            if target[index].flag:
                conflict_hunk_num_list.append(change.hunk)

            del target[index]
            del_count += 1

        else:
            # 对其他行也要标记 flag
            index = change.old - 1 - del_count + add_count

            try:
                assert index == change.new - 1  # TODO: but why? 44733
            except AssertionError:
                logger.error(
                    f"index: {index}, change.new: {change.new}, hunk: {change.hunk}, patch: {patch_path.split('/')[-1][:6]}"
                )

            target[index].flag = (
                True if flag and change.hunk in flag_hunk_list else target[index].flag
            )  # 加点注释解释一下 # TODO: 确认这个条件是否是正确的

    new_line_list: list[Line] = []
    for index, line in enumerate(target):
        # 判断是否在 Flag 行附近进行了修改
        # 如果该行为 changed，且前后行为flag，则也加入标记列表
        if flag and line.changed and not line.flag:
            before_flag = (
                index > 0 and target[index - 1].flag and not target[index - 1].changed
            )
            after_flag = (
                index < len(target) - 1
                and target[index + 1].flag
                and not target[index + 1].changed
            )

            if before_flag or after_flag:
                line.flag = True

            if line.flag:
                conflict_hunk_num_list.append(line.hunk)

            # 当 trace 为 True 时，将所有 conflict hunk 的行都标记为冲突
            if trace and line.hunk in conflict_hunk_num_list:
                line.flag = True

        new_line_list.append(
            Line(index=index, content=line.content, flag=line.flag, hunk=line.hunk)
        )  # 洗掉 changed 注意在下面洗掉 hunk

    # WIP: 在 conclict hunk 中搜索 symbol，注意去重
    logger.debug("patch_path: " + patch_path)
    if symbols is not None and len(conflict_hunk_num_list) != 0:
        logger.debug(f"Searching symbol in conflict hunk")
        # patch_path 已经是筛选后的 patch，仅包含 filename 对应内容
        # patch_path 仅有一个 diff，故只获取首个 diff 即可
        symbol_results = getsymbol_from_patch(patch_path, symbols)
        extra_hunks = next(iter(symbol_results.values()), [])
        logger.debug(f"Extra hunk list: {extra_hunks}")

        # 合并 extra_hunks 到 apply_result.conflict_hunk_num_list
        conflict_hunk_num_list += extra_hunks

        for line in new_line_list:
            if line.hunk in conflict_hunk_num_list:
                line.flag = True
            line.hunk = None
    else:
        for line in new_line_list:
            line.hunk = None

    failed_hunk_list.extend(
        [
            hunk
            for hunk in hunk_list
            if hunk.index in conflict_hunk_num_list and hunk not in failed_hunk_list
        ]
    )

    return ApplyResult(
        new_line_list=new_line_list,
        conflict_hunk_num_list=conflict_hunk_num_list,
        failed_hunk_list=failed_hunk_list,
    )
